Attribute VB_Name = "mPeristalis"
Option Explicit

'//------------------------------------------------------------------------
'//
'// Author      : Jan Horn
'// Email       : jhorn@global.co.za
'// Website     : http://home.global.co.za/~jhorn
'// Date        : 14 April 2001
'// Version     : 1.0
'// Description : Peristalsis
'// Translate   : from Delphi to VisualBasic " Emil Weiss " emilweiss@t-online.de
'//------------------------------------------------------------------------
'program Peristalsis;

Private Declare Function EnumDisplaySettings Lib "user32" Alias "EnumDisplaySettingsA" (ByVal lpszDeviceName As Long, ByVal iModeNum As Long, lpDevMode As Any) As Boolean
Private Declare Function ChangeDisplaySettings Lib "user32" Alias "ChangeDisplaySettingsA" (lpDevMode As Any, ByVal dwflags As Long) As Long
Private Declare Function CreateIC Lib "gdi32" Alias "CreateICA" (ByVal lpDriverName As String, ByVal lpDeviceName As String, ByVal lpOutput As String, ByVal lpInitData As Long) As Long


Private Const CCDEVICENAME = 32
Private Const CCFORMNAME = 32
'Private Const DM_BITSPERPEL = &H40000
'Private Const DM_PELSWIDTH = &H80000
'Private Const DM_PELSHEIGHT = &H100000

' DEVMODE declare used to query Windows for the
' Display device settings
' the one in the typelib has problems so I included this one.
Private Type DEVMODE
    dmDeviceName        As String * CCDEVICENAME
    dmSpecVersion       As Integer
    dmDriverVersion     As Integer
    dmSize              As Integer
    dmDriverExtra       As Integer
    dmFields            As Long
    dmOrientation       As Integer
    dmPaperSize         As Integer
    dmPaperLength       As Integer
    dmPaperWidth        As Integer
    dmScale             As Integer
    dmCopies            As Integer
    dmDefaultSource     As Integer
    dmPrintQuality      As Integer
    dmColor             As Integer
    dmDuplex            As Integer
    dmYResolution       As Integer
    dmTTOption          As Integer
    dmCollate           As Integer
    dmFormName          As String * CCFORMNAME
    dmUnusedPadding     As Integer
    dmBitsPerPel        As Integer
    dmPelsWidth         As Long
    dmPelsHeight        As Long
    dmDisplayFlags      As Long
    dmDisplayFrequency  As Long
End Type

Public Keys(255) As Boolean             ' used to keep track of key_downs

Private hrc As Long                     ' handle for OpenGL redering context
Private fullscreen As Boolean           ' are we in full screen

Private OldWidth As Long                ' used to restore the starting display settings
Private OldHeight As Long               ' Height
Private OldBits As Long                 ' bit per pixel
Private OldVertRefresh As Long          ' and vertical refresh

Private mPointerCount As Integer        ' maintains the number of show/hide cursor calls

Public Const MAX_SEGMENTS = 64
Public Const MAX_DIVS = 16
Public ElapsedTime As Long                  '// Elapsed time between frames
Public LastTime As Long
Public DemoStart As Long
  
'// Textures
Public TubeTex As cTexture
Public BackTex As cTexture

'// User vaiables
Public twist(MAX_SEGMENTS) As GLfloat
Public radius(0 To MAX_SEGMENTS) As GLfloat
Public xpos(0 To MAX_SEGMENTS) As GLfloat
Public Const Pi = 3.14159265358979
Public m_Texture As cTexture                '// Our own texture

Private Sub HidePointer()
    '// hide the cursor (mouse pointer)
    mPointerCount = ShowCursor(False) + 1   '// how many times will we need to hide the pointer
    Do While ShowCursor(False) >= -1        '// make sure it's hidden
    Loop
    Do While ShowCursor(True) <= -1         '// make sure the next call will show it
    Loop
    ShowCursor False                        '// one last time
End Sub

Private Sub ShowPointer()
    '// show the cursor (mouse pointer)
    Do While ShowCursor(False) >= mPointerCount '// restore the cursor back to normal
    Loop
    Do While ShowCursor(True) <= mPointerCount
    Loop
End Sub

Public Sub ReSizeGLScene(ByVal Width As GLsizei, ByVal Height As GLsizei)
    '// Resize And Initialize The GL Window
    If Height = 0 Then                          '// Prevent A Divide By Zero By
        Height = 1                              '// Making Height Equal One
    End If
    glViewport 0, 0, Width, Height              '// Reset The Current Viewport
    glMatrixMode GL_PROJECTION                  '// Select The Projection Matrix
    glLoadIdentity                              '// Reset The Projection Matrix

    '// Calculate The Aspect Ratio Of The Window
    gluPerspective 45#, Width / Height, 0.1, 100#

    glMatrixMode GL_MODELVIEW                   '// Select The Modelview Matrix
    glLoadIdentity                              '// Reset The Modelview Matrix
End Sub

Public Function InitGL() As Boolean
Dim i As Integer
  Set TubeTex = New cTexture
  Set BackTex = New cTexture
  Call glClearColor(0#, 0#, 0#, 0#)            '// Black Background
  Call glShadeModel(GL_SMOOTH)                 '// Enables Smooth Color Shading
  Call glClearDepth(1#)                        '// Depth Buffer Setup
  Call glEnable(GL_DEPTH_TEST)                 '// Enable Depth Buffer
  Call glDepthFunc(GL_LESS)                    '// The Type Of Depth Test To Do

  Call glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)   '//Realy Nice perspective calculations

  Call glEnable(GL_TEXTURE_2D)                             '// Enable Texture Mapping
  Call TubeTex.loadTexture(App.Path & "\Data\" & "images\Tube.bmp", FILETYPE_BMP)
  Call BackTex.loadTexture(App.Path & "\Data\" & "images\background.bmp", FILETYPE_BMP)
  For i = 0 To MAX_SEGMENTS
    twist(i) = 0
    radius(i) = 1
  Next i
  
  InitGL = True
End Function


Public Sub KillGLWindow()
' Properly Kill The Window
    If fullscreen Then                              ' Are We In Fullscreen Mode?
        ResetDisplayMode                            ' If So Switch Back To The Desktop
        ShowPointer                                 ' Show Mouse Pointer
    End If

    If hrc Then                                     ' Do We Have A Rendering Context?
        If wglMakeCurrent(0, 0) = 0 Then             ' Are We Able To Release The DC And RC Contexts?
            MsgBox "Release Of DC And RC Failed.", vbInformation, "SHUTDOWN ERROR"
        End If

        If wglDeleteContext(hrc) = 0 Then           ' Are We Able To Delete The RC?
            MsgBox "Release Rendering Context Failed.", vbInformation, "SHUTDOWN ERROR"
        End If
        hrc = 0                                     ' Set RC To NULL
    End If

    ' Note
    ' The form owns the device context (hDC) window handle (hWnd) and class (RTThundermain)
    ' so we do not have to do all the extra work

End Sub

Private Sub SaveCurrentScreen()
    '// Save the current screen resolution, bits, and Vertical refresh
    Dim ret As Long
    ret = CreateIC("DISPLAY", "", "", 0&)           '// create display information context
    OldWidth = GetDeviceCaps(ret, HORZRES)          '// get the current width
    OldHeight = GetDeviceCaps(ret, VERTRES)         '// and current height
    OldBits = GetDeviceCaps(ret, BITSPIXEL)         '// and bits per pixel (color depth)
    OldVertRefresh = GetDeviceCaps(ret, VREFRESH)   '// and current refresh rate (this may cause problems in Win98
    ret = DeleteDC(ret)                             '// delete the information context
End Sub

Private Function FindDEVMODE(ByVal Width As Integer, ByVal Height As Integer, ByVal Bits As Integer, Optional ByVal VertRefresh As Long = -1) As DEVMODE
    '// locate a DEVMOVE that matches the passed parameters
    Dim ret As Boolean      '// return value from api
    Dim i As Long           '// loop counter
    Dim dm As DEVMODE       '// Device mode
    i = 0
    Do  '// enumerate the display settings until we find the one we want
        ret = EnumDisplaySettings(0&, i, dm)    '// get windows to enumerate the display settings
        If dm.dmPelsWidth = Width And _
            dm.dmPelsHeight = Height And _
            dm.dmBitsPerPel = Bits And _
            ((dm.dmDisplayFrequency = VertRefresh) Or (VertRefresh = -1)) Then Exit Do ' exit when we have a match
        i = i + 1
    Loop Until (ret = False)
    FindDEVMODE = dm
End Function

Private Sub ResetDisplayMode()
    Dim dm As DEVMODE             '// Device Mode
    
    dm = FindDEVMODE(OldWidth, OldHeight, OldBits, OldVertRefresh)  '// find our old display mode
    dm.dmFields = DM_BITSPERPEL Or DM_PELSWIDTH Or DM_PELSHEIGHT    '// what we are about to change
    If OldVertRefresh <> -1 Then
        dm.dmFields = dm.dmFields Or DM_DISPLAYFREQUENCY            '// include vertical refresh
    End If
    '// Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
    If (ChangeDisplaySettings(dm, CDS_FULLSCREEN) <> DISP_CHANGE_SUCCESSFUL) Then
    
        '// If The Mode Fails, Offer Two Options.  Quit Or Run In A Window.
        MsgBox "The Requested Mode Is Not Supported By Your Video Card", , "NeHe GL"
    End If

End Sub

Private Sub SetDisplayMode(ByVal Width As Integer, ByVal Height As Integer, ByVal Bits As Integer, ByRef fullscreen As Boolean, Optional VertRefresh As Long = -1)
    Dim dmScreenSettings As DEVMODE             '// Device Mode
    Dim p As Long
    SaveCurrentScreen                           '// save the current screen attributes so we can go back later
    
    dmScreenSettings = FindDEVMODE(Width, Height, Bits, VertRefresh) '// find a device mode that matches
    dmScreenSettings.dmBitsPerPel = Bits    '// set pixel bits
    dmScreenSettings.dmPelsWidth = Width    '// width
    dmScreenSettings.dmPelsHeight = Height  '// height
    dmScreenSettings.dmFields = DM_PELSWIDTH Or DM_PELSHEIGHT Or DM_BITSPERPEL ' what we are going to change
    If VertRefresh <> -1 Then
        dmScreenSettings.dmDisplayFrequency = VertRefresh '// include vertical refresh
        dmScreenSettings.dmFields = dmScreenSettings.dmFields Or DM_DISPLAYFREQUENCY
    End If
    '// Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
    If (ChangeDisplaySettings(dmScreenSettings, CDS_FULLSCREEN) <> DISP_CHANGE_SUCCESSFUL) Then
    
        '// If The Mode Fails, Offer Two Options.  Quit Or Run In A Window.
        If (MsgBox("The Requested Mode Is Not Supported By" & vbCr & "Your Video Card. Use Windowed Mode Instead?", vbYesNo + vbExclamation, "NeHe GL") = vbYes) Then
            fullscreen = False                  '// Select Windowed Mode (Fullscreen=FALSE)
        Else
            '// Pop Up A Message Box Letting User Know The Program Is Closing.
            MsgBox "Program Will Now Close.", vbCritical, "ERROR"
            End                   '// Exit And Return FALSE
        End If
    End If
End Sub

Public Function DrawGLScene() As Boolean
Dim i As Integer
Dim J As Integer
Dim Angle As GLfloat
  Call glClear(GL_COLOR_BUFFER_BIT Or GL_DEPTH_BUFFER_BIT)    '// Clear The Screen And The Depth Buffer

  Call glLoadIdentity                                         '// Reset The View
  Angle = ElapsedTime / 300

  '//--- Background effect ---//
  Call glPushMatrix
  Call glTranslatef(0, 0, -5)
  Call glRotatef(25 * Angle, 1, 0, 0)
  Call glRotatef(45 * Angle, 0, 1, 0)

  Call glScalef(5, 5, 5)
  BackTex.useTexture
  Call glBegin(GL_QUADS)
      '// Front Face
  Call glNormal3f(0#, 0#, 1#)
  Call glTexCoord2f(0#, 0#): Call glVertex3f(-1#, -1#, 1#)
  Call glTexCoord2f(1#, 0#): Call glVertex3f(1#, -1#, 1#)
  Call glTexCoord2f(1#, 1#): Call glVertex3f(1#, 1#, 1#)
  Call glTexCoord2f(0#, 1#): Call glVertex3f(-1#, 1#, 1#)
      '// Back Face
  Call glNormal3f(0#, 0#, -1#)
  Call glTexCoord2f(1#, 0#): Call glVertex3f(-1#, -1#, -1#)
  Call glTexCoord2f(1#, 1#): Call glVertex3f(-1#, 1#, -1#)
  Call glTexCoord2f(0#, 1#): Call glVertex3f(1#, 1#, -1#)
  Call glTexCoord2f(0#, 0#): Call glVertex3f(1#, -1#, -1#)
      '// Top Face
  Call glNormal3f(0#, 1#, 0#)
  Call glTexCoord2f(0#, 1#): Call glVertex3f(-1#, 1#, -1#)
  Call glTexCoord2f(0#, 0#): Call glVertex3f(-1#, 1#, 1#)
  Call glTexCoord2f(1#, 0#): Call glVertex3f(1#, 1#, 1#)
  Call glTexCoord2f(1#, 1#): Call glVertex3f(1#, 1#, -1#)
      '// Bottom Face
  Call glNormal3f(0#, -1#, 0#)
  Call glTexCoord2f(1#, 1#): Call glVertex3f(-1#, -1#, -1#)
  Call glTexCoord2f(0#, 1#): Call glVertex3f(1#, -1#, -1#)
  Call glTexCoord2f(0#, 0#): Call glVertex3f(1#, -1#, 1#)
  Call glTexCoord2f(1#, 0#): Call glVertex3f(-1#, -1#, 1#)
      '// Right face
  Call glNormal3f(1#, 0#, 0#)
  Call glTexCoord2f(1#, 0#): Call glVertex3f(1#, -1#, -1#)
  Call glTexCoord2f(1#, 1#): Call glVertex3f(1#, 1#, -1#)
  Call glTexCoord2f(0#, 1#): Call glVertex3f(1#, 1#, 1#)
  Call glTexCoord2f(0#, 0#): Call glVertex3f(1#, -1#, 1#)
      '// Left Face
  Call glNormal3f(-1#, 0#, 0#)
  Call glTexCoord2f(0#, 0#): Call glVertex3f(-1#, -1#, -1#)
  Call glTexCoord2f(1#, 0#): Call glVertex3f(-1#, -1#, 1#)
  Call glTexCoord2f(1#, 1#): Call glVertex3f(-1#, 1#, 1#)
  Call glTexCoord2f(0#, 1#): Call glVertex3f(-1#, 1#, -1#)
  Call glEnd
  Call glPopMatrix

  TubeTex.useTexture
  Call glTranslatef(0, 0, -4.6)
  Call glRotatef(Angle * 4, 0, 0, 1)
  Call glTranslatef(0, -2.75, 0)
  On Error Resume Next

  '//--- DrawCylinder ---//
  Call glBegin(GL_QUADS)
    For i = 0 To MAX_SEGMENTS ' - 1
      For J = 0 To MAX_DIVS
        '//  i,j
        Call glTexCoord2f(twist(i) + J * 4 / MAX_DIVS, i * 5.5 / MAX_SEGMENTS)
        Call glVertex3f(xpos(i) + 0.7 * radius(i) * Cos(J * Pi * 2# / MAX_DIVS), i * 5.5 / MAX_SEGMENTS, 0.7 * radius(i) * Sin(J * Pi * 2# / MAX_DIVS))
        
        '//  i+1, j
        Call glTexCoord2f(twist(i + 1) + J * 4 / MAX_DIVS, (i + 1) * 5.5 / MAX_SEGMENTS)
        Call glVertex3f(xpos(i + 1) + 0.7 * radius(i + 1) * Cos(J * Pi * 2# / MAX_DIVS), (i + 1) * 5.5 / MAX_SEGMENTS, 0.7 * radius(i + 1) * Sin(J * Pi * 2# / MAX_DIVS))

        '//  i+1, j+1
        Call glTexCoord2f(twist(i + 1) + (J + 1) * 4 / MAX_DIVS, (i + 1) * 5.5 / MAX_SEGMENTS)
        Call glVertex3f(xpos(i + 1) + 0.7 * radius(i + 1) * Cos((J + 1) * Pi * 2# / MAX_DIVS), (i + 1) * 5.5 / MAX_SEGMENTS, 0.7 * radius(i + 1) * Sin((J + 1) * Pi * 2# / MAX_DIVS))

       '//  i, j+1
        Call glTexCoord2f(twist(i) + (J + 1) * 4 / MAX_DIVS, i * 5.5 / MAX_SEGMENTS)
        Call glVertex3f(xpos(i) + 0.7 * radius(i) * Cos((J + 1) * Pi * 2# / MAX_DIVS), i * 5.5 / MAX_SEGMENTS, 0.7 * radius(i) * Sin((J + 1) * Pi * 2# / MAX_DIVS))
      Next J
    Next i
  Call glEnd

  '// Recalculate values
  For i = 0 To MAX_SEGMENTS ' - 1
    twist(i) = 0.35 * Sin(Angle * 0.78 + i / 12#) + 0.35 * Sin(Angle * -1.23 + i / 18#)
    radius(i) = 0.3 * Sin(Angle + i / 8#) + 0.8 + 0.15 * Sin(Angle * -0.8 + i / 3#)
    xpos(i) = 0.25 * Sin(Angle * 1.23 + i / 5#) + 0.3 * Sin(Angle * 0.9 + i / 6#)
  Next i

  DrawGLScene = True
End Function

Public Function CreateGLWindow(frm As Form, Width As Integer, Height As Integer, Bits As Integer, fullscreenflag As Boolean) As Boolean
    Dim PixelFormat As GLuint                       ' Holds The Results After Searching For A Match
    Dim pfd As PIXELFORMATDESCRIPTOR                ' pfd Tells Windows How We Want Things To Be


    fullscreen = fullscreenflag                     ' Set The Global Fullscreen Flag


    If (fullscreen) Then                            ' Attempt Fullscreen Mode?
        SetDisplayMode Width, Height, Bits, fullscreen
    End If
    
    If fullscreen Then
        HidePointer                                 ' Hide Mouse Pointer
        frm.WindowState = vbMaximized           ' make the form big
    End If

    pfd.cColorBits = Bits               ' color depth
    pfd.cDepthBits = 16
    pfd.dwflags = PFD_DRAW_TO_WINDOW Or PFD_SUPPORT_OPENGL Or PFD_DOUBLEBUFFER
    pfd.iLayerType = PFD_MAIN_PLANE
    pfd.iPixelType = PFD_TYPE_RGBA
    pfd.nSize = Len(pfd)
    pfd.nVersion = 1
    
    PixelFormat = ChoosePixelFormat(frm.hDC, pfd)
    If PixelFormat = 0 Then                     ' Did Windows Find A Matching Pixel Format?
        KillGLWindow                            ' Reset The Display
        MsgBox "Can't Find A Suitable PixelFormat.", vbExclamation, "ERROR"
        CreateGLWindow = False                  ' Return FALSE
    End If

    If SetPixelFormat(frm.hDC, PixelFormat, pfd) = 0 Then ' Are We Able To Set The Pixel Format?
        KillGLWindow                            ' Reset The Display
        MsgBox "Can't Set The PixelFormat.", vbExclamation, "ERROR"
        CreateGLWindow = False                           ' Return FALSE
    End If
    
    hrc = wglCreateContext(frm.hDC)
    If (hrc = 0) Then                           ' Are We Able To Get A Rendering Context?
        KillGLWindow                            ' Reset The Display
        MsgBox "Can't Create A GL Rendering Context.", vbExclamation, "ERROR"
        CreateGLWindow = False                  ' Return FALSE
    End If

    If wglMakeCurrent(frm.hDC, hrc) = 0 Then    ' Try To Activate The Rendering Context
        KillGLWindow                            ' Reset The Display
        MsgBox "Can't Activate The GL Rendering Context.", vbExclamation, "ERROR"
        CreateGLWindow = False                  ' Return FALSE
    End If
    frm.Show                                    ' Show The Window
    SetForegroundWindow frm.hWnd                ' Slightly Higher Priority
    frm.SetFocus                                ' Sets Keyboard Focus To The Window
    ReSizeGLScene frm.ScaleWidth, frm.ScaleHeight ' Set Up Our Perspective GL Screen

    If Not InitGL() Then                        ' Initialize Our Newly Created GL Window
        KillGLWindow                            ' Reset The Display
        MsgBox "Initialization Failed.", vbExclamation, "ERROR"
        CreateGLWindow = False                   ' Return FALSE
    End If

    CreateGLWindow = True                       ' Success

End Function

Sub Main()
    Dim Done As Boolean
    Dim frm As Form
    Done = False ' we're not done yet
    ' Ask The User Which Screen Mode They Prefer
    fullscreen = MsgBox("Would You Like To Run In Fullscreen Mode?", vbYesNo + vbQuestion, "Start FullScreen?") = vbYes
    DemoStart = GetTickCount

    ' Create Our OpenGL Window
    Set frm = New Form1     ' create our form
    If Not CreateGLWindow(frm, 640, 480, 16, fullscreen) Then
        Done = True                             ' Quit If Window Was Not Created
    End If

    Do While Not Done
        ' Draw The Scene.  Watch For ESC Key And Quit Messages From DrawGLScene()
        If (Not DrawGLScene Or Keys(vbKeyEscape)) Then  ' Updating View Only If Active
            Unload frm                          ' ESC or DrawGLScene Signalled A Quit
        Else                                    ' Not Time To Quit, Update Screen
            LastTime = ElapsedTime
            ElapsedTime = GetTickCount() - DemoStart    '// Calculate Elapsed Time
            ElapsedTime = (LastTime + ElapsedTime) \ 2  '// Average it out for smoother movement
            SwapBuffers (frm.hDC)               ' Swap Buffers (Double Buffering)
            DoEvents
        End If
        If Keys(vbKeyW) Then
            Call glPolygonMode(GL_FRONT, GL_LINE)
            Call glPolygonMode(GL_BACK, GL_LINE)
        End If
        If Keys(vbKeyT) Then
           Call glPolygonMode(GL_FRONT, GL_FILL)
           Call glPolygonMode(GL_BACK, GL_FILL)
        End If
        If Keys(vbKeyF1) Then                   ' Is F1 Being Pressed?
            Keys(vbKeyF1) = False               ' If So Make Key FALSE
            Unload frm                          ' Kill Our Current Window
            Set frm = New Form1                 ' create a new one
            fullscreen = Not fullscreen         ' Toggle Fullscreen / Windowed Mode
            ' Recreate Our OpenGL Window
            If Not CreateGLWindow(frm, 640, 480, 16, fullscreen) Then
                Unload frm                      ' Quit If Window Was Not Created
            End If
        End If
        Done = frm.Visible = False              ' if the form is not visible then we are done
    Loop
    ' Shutdown
    Set frm = Nothing ' destroy our form
    End
End Sub
